@@ -73,7 +73,8 @@ static GConfEnumStringPair panel_action_type_map [] = {
 	{ PANEL_ACTION_SEARCH,         "search"         },
 	{ PANEL_ACTION_SCREENSHOT,     "screenshot"     },
 	{ PANEL_ACTION_FORCE_QUIT,     "force-quit"     },
-	{ PANEL_ACTION_CONNECT_SERVER, "connect-server" }
+	{ PANEL_ACTION_CONNECT_SERVER, "connect-server" },
+	{ 0,                           NULL             },
 };
 
 /* Lock Screen
@@ -914,11 +915,9 @@ panel_action_button_set_dnd_enabled (PanelActionButton *button,
 		gtk_drag_source_set (GTK_WIDGET (button), GDK_BUTTON1_MASK,
 				     dnd_targets, 1,
 				     GDK_ACTION_COPY | GDK_ACTION_MOVE);
-		/* FIXME: waiting for bug #116577
 		if (actions [button->priv->type].icon_name != NULL)
 		gtk_drag_source_set_icon_name (GTK_WIDGET (button),
-						actions [button->priv->type].icon_name);
-						*/
+					       actions [button->priv->type].icon_name);
 		GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
 	} else
 		gtk_drag_source_unset (GTK_WIDGET (button));
